/*
 * =====================================================================================
 * 
 *        Filename:  testImage.cc
 * 
 *     Description:  testing template  Image Class
 * 
 *         Version:  1.0
 *         Created:  05/10/05 14:07:10 CEST
 *        Revision:  none
 *        Compiler:  gcc
 * 
 *          Author:  Janis Fehr (JF), fehr@informatiik.uni-freiburg.de
 *         Company:  LMB
 * 
 * =====================================================================================
 */

#include <iostream>
#include "Image.hh"

        

/*======================================================================*/
/*!
 *   Check if the default constructor works.
 */
/*======================================================================*/
static bool testEmptyContructor()
{
  std::cout << "testEmptyContructor\n";

  Image<int> image;

  if (image.width()!=0 || image.height()!=0 ||
      image.size()!=0 ||
      image.begin()!=image.end())
  {
    std::cout << "[properties] failed\n";
    return false;
  }

  std::cerr << "ok\n";
  return true;
}


/*======================================================================*/
/*!
 *   Check if the copy constructor works.
 */
/*======================================================================*/
static bool testCopyConstructor()
{
  std::cerr << "testCopyConstructor\n";

  // create 0 image
  Image<int> image1(5, 5);
  for (int* p=image1.begin();
       p!=image1.end();
       ++p)
  {
    *p=0;
  }

  // copy image
  Image<int> image2(image1);

  // check if both images have equal properties
  if (image1.width()!=image2.width() ||
      image1.height()!=image2.height() ||
      image1.size()!=image2.size())
  {
    std::cerr << "[properties] failed\n";
    return false;
  }

  // check if pointers are different
  if (image1.begin()==image2.begin() ||
      image1.end()==image2.end())
  {
    std::cerr << "[pointers] failed\n";
    return false;
  }

  // check if copying also copied pixels
  for (const int* p=image2.begin();
       p!=image2.end();
       ++p)
  {
    if (*p!=0)
    {
      std::cerr << "[pixels] failed\n";
      return false;
    }
  }

  // change one pixel in new image
  image2(3, 3)=1;

  // check if pixel changed not in original image but in new one
  if (image1(3, 3)!=0 || image2(3, 3)!=1)
  {
    std::cerr << "[allocation] failed\n";
    return false;
  }

  std::cerr << "ok\n";
  return true;
}

/*======================================================================*/
/*!
 *   Check if the size constructor works.
 */
/*======================================================================*/
static bool testSizeConstructor()
{
  std::cout << "testSizeConstructor\n";

  // create image of arbitrary size
  Image<int> image(42, 17);
  // check if properties are correct
  if (image.width()!=42 || image.height()!=17 || image.size()!=42*17 ||
      image.begin()+image.size()!=image.end())
  {
    std::cerr << "[properties] failed\n";
    return true;
  }

  std::cerr << "ok\n";
  return true;
}

/*======================================================================*/
/*!
 *   Check if resize() works.
 */
/*======================================================================*/
static bool testResize()
{
  std::cout << "testResize\n";

  // create image
  Image<int> image(42, 17);
  // resize image
  image.resize(117, 5);
  // check if properties are correct
  if (image.width()!=117 || image.height()!=5 || image.size()!=117*5 ||
      image.begin()+image.size()!=image.end())
  {
    std::cerr << "[properties] failed\n";
    return true;
  }

  // set color of arbitrary pixel
  image(116, 4)=0;
  // try to access that pixel by op[]
  if (image[117*5-1]!=0)
  {
    std::cerr << "[pixel access] failed\n";
    return true;
  }

  // try to make image really huge
  try
  {
    image.resize(1000000, 1000000);
  }
  catch (const MyError& e) // correct exception thrown
  {
  }
  catch (...) // probably bad_alloc exception
  {
    std::cerr << "[oversize] failed\n";
    return false;
  }

  std::cerr << "ok\n";
  return true;
}

/*======================================================================*/
/*!
 *   Check if operator=() works, see testCopyConstructor()
 */
/*======================================================================*/
static bool testAssignmentOperator()
{
  std::cerr << "testAssignmentOperator\n";

  Image<int> image1(5, 5);
  for (int* p=image1.begin();
       p!=image1.end();
       ++p)
  {
    *p=0;
  }

  Image<int> image2(3, 3);
  image2=image1;

  if (image1.width()!=image2.width() ||
      image1.height()!=image2.height() ||
      image1.size()!=image2.size())
  {
    std::cerr << "[properties] failed\n";
    return false;
  }

  if (image1.begin()==image2.begin() ||
      image1.end()==image2.end())
  {
    std::cerr << "[pointers] failed\n";
    return false;
  }

  for (const int* p=image2.begin();
       p!=image2.end();
       ++p)
  {
    if (*p!=0)
    {
      std::cerr << "[pixels] failed\n";
      return false;
    }
  }

  for (int* p=image2.begin();
       p!=image2.end();
       ++p)
  {
    *p=1;
  }


  if (image1(3, 3)!=0 || image2(3, 3)!=1)
  {
    std::cerr << "[allocation] failed\n";
    return false;
  }

  std::cerr << "ok\n";
  return true;
}

/*======================================================================*/
/*!
 *   Check if read() works.
 */
/*======================================================================*/
static bool testRead()
{
  std::cerr << "testRead\n";

  Image<RGBColor> image;

  std::ifstream file1("16x16_red_gradient.ppm");
  image.read(file1);

  // check properties
  if (image.width()!=16 || image.height()!=16 ||
      image[0]!=RGBColor::Red || image[image.size()-1]!=RGBColor::White)
  {
    std::cerr << "[load correct] failed\n";
    return false;
  }

  // read file with many comments
  bool ok=true;
  std::ifstream file2("16x16_red_gradient.ppm");
  try
  {
    image.read(file2);
  }
  catch (const MyError& e)
  {
    ok=false;
  }
  catch (...)
  {
    ok=false;
  }

  if (!ok)
  {
    std::cerr << "[load comments] failed\n";
    return false;
  }


  // read file with invalid header
  ok=false;
  std::ifstream file3("badheader.ppm");
  try
  {
    image.read(file3);
  }
  catch (const MyError& e)
  {
    ok=true;
  }
  catch (...)
  {
    ok=false;
  }

  if (!ok)
  {
    std::cerr << "[load badheader] failed\n";
    return false;
  }

  std::cerr << "ok\n";
  return true;

}

/*======================================================================*/
/*!
 *   Check if write() works.
 */
/*======================================================================*/
static bool testWrite()
{
  std::cerr << "testWrite\n";

  // create image
  Image<RGBColor> image(16, 16);
  // set pixel colors
  for (unsigned int y=0; y<image.height(); ++y)
  {
    for (unsigned int x=0; x<image.width(); ++x)
    {
      image(x,y).set(x, 0, y);
    }
  }
  // write image
  std::ofstream file("out.ppm");
  image.write(file);
  file.flush();

  std::ifstream filein("out.ppm");
  Image<RGBColor> image2;
  image2.read(filein);

  if (image.size()!=image2.size() ||
      image.width()!=image2.width() ||
      image.height()!=image2.height())
  {
    std::cerr << "[properties failed]\n";
    return false;
  }

  for (unsigned int y=0; y<image.height(); ++y)
  {
    for (unsigned int x=0; x<image.width(); ++x)
    {
      if (image(x,y)!=image2(x,y))
      {
        std::cerr << "[pixels failed]\n";
        return false;
      }
    }
  }

  std::cerr << "ok\n";
  return true;
}


int main()
{
 try
  {
    testEmptyContructor();
    testCopyConstructor();
    testSizeConstructor();
    testResize();
    testAssignmentOperator();
    testRead();
    testWrite();
  }
  catch (const MyError& e)
  {
    std::cerr << "MyError caught: " << e.msg() << std::endl;
  }
  catch (...)
  {
    std::cerr << "unknown exception caught\n";
  }

	

}



